@@ -19,6 +19,8 @@ try:
   from _thread import start_new_thread
 except:
   from thread import start_new_thread
+from sys import version_info
+PY3 = version_info.major >= 3
 
 logger = getLogger(__name__)
 
@@ -49,6 +51,15 @@ class _Schedule(object):
   def get_result(self): return self._result
 
 
+if PY3:
+  class AtomicItemsDict(dict):
+    def items(self):
+      """return items as a list in a thread safe way."""
+      while True:
+        try: return list(super().items())
+        except RuntimeError: pass
+else: AtomicItemsDict = dict
+
 class TransactionalScheduler(object):
   """scheduler for asynchronous operations and their management.
 
@@ -62,7 +73,7 @@ class TransactionalScheduler(object):
   timeout = 3600 # s; delete completions older than this (forgotten requests)
 
   # may be overridden by dezived classes
-  MAPPING_FACTORY = dict
+  MAPPING_FACTORY = AtomicItemsDict
   SCHEDULE_FACTORY = _Schedule
 
   def __init__(self, timeout=None):
